#!/usr/bin/bash
function help()
{
    echo "Usage: $(basename $0) iso_in iso_out"
    echo "       iso_in   input LiveOS ISO image"
    echo "       iso_out  output LiveOS ISO image"
}

if [ $EUID -ne 0 ]; then
    echo "Error: this script must be run as root"
    exit 1
fi
if [ $# -lt 2 ]; then
    echo "Error: not enough arguments"
    help "$@"
    exit 1
fi
iso_in="$1"
iso_out="$2"
wd=$(mktemp -d --tmpdir=. iso.XXX) #working directory
volid=$(isoinfo -i "$iso_in" -d | sed -n 's/Volume id: //p')

echo "=== Creating working directories"
mkdir -p "$wd/mnt"
mkdir -p "$wd/files"
mkdir -p "$wd/efiboot"
mkdir -p "$wd/squashfs"

echo "=== Extracting files from the LiveOS image"
mount -o ro "$iso_in" "$wd/mnt"
rsync -aP "$wd/mnt/" "$wd/files"
umount "$wd/mnt"

echo "=== Determining image type: "
if [ -d "$wd/files/LiveOS" ]; then
    echo "LiveOS"
    liveimg=1
    squashfspath="LiveOS/squashfs.img"
else
    echo "netinstall"
    liveimg=0
    squashfspath="images/install.img"
fi

echo "=== Extracting the root partition from squashfs"
mount -o ro "$wd/files/$squashfspath" "$wd/mnt"
rsync -aP "$wd/mnt/" "$wd/squashfs"
umount "$wd/mnt"

echo "=== Mounting the root partition and efiboot image"
mount "$wd/squashfs/LiveOS/rootfs.img" "$wd/mnt"
mount "$wd/files/images/efiboot.img" "$wd/efiboot"

echo "=== Copying setup script to LiveOS root partition"
mkdir -p "$wd/mnt/usr/local/share"
rsync -a gpd-pocket-setup "$wd/mnt/usr/local/share"

if [ $liveimg -eq 1 ]; then
    echo "=== Preparing chroot"
    for m in {proc,sys,dev,dev/pts}; do
        mount -o bind "/$m" "$wd/mnt/$m"
    done
    mv "$wd/mnt/etc/resolv.conf"{,.backup} 2>/dev/null
    cp /etc/resolv.conf "$wd/mnt/etc/resolv.conf"
    mkdir -p "$wd/mnt/boot"

    echo "=== Executing setup script under chroot"
    chroot "$wd/mnt" bash -c "cd /usr/local/share/gpd-pocket-setup; ./run.sh --live"

    echo "=== Cleaning up chroot"
    for m in {proc,sys,dev/pts,dev}; do
        umount "$wd/mnt/$m"
    done
    rm -f "$wd/mnt/etc/resolv.conf"
    mv "$wd/mnt/etc/resolv.conf"{.backup,} 2>/dev/null

    echo "=== Retrieving kernel and init ram disk filenames"
    kernelfn=$(find "$wd/mnt/boot" -name "vmlinuz*gpdpocket*" -exec basename "{}" \;)
    initrdfn=$(find "$wd/mnt/boot" -name "initr*gpdpocket*.img" -exec basename "{}" \;)

    echo "=== Copying kernel and init ram disk to LiveOS efiboot image"
    rsync -a "$wd/mnt/boot/$kernelfn" "$wd/files/images/pxeboot/"
    mv "$wd/mnt/usr/local/share/gpd-pocket-setup/$initrdfn" "$wd/files/images/pxeboot/"

    echo "=== Modyfing grub config"
    sed -i "0,/menuentry/{
        /menuentry/{
            :a
            N;s/\}/\}/;T a
            p;h
            s/\('.*\)'/\1 Hans kernel'/
            s|/vmlinuz|/$kernelfn|
            s|/initrd.img|/$initrdfn|
            p;g
            s/\('.*\)'/\1 Hans kernel kickstart install'/
            s|/vmlinuz|/$kernelfn|
            s|/initrd.img|/$initrdfn|
            s|\(linuxefi[^\n]*\)\n|\1 ks=hd:$volid/ks.cfg\n|
        }}" "$wd/efiboot/EFI/BOOT/grub.cfg"
else
    echo "=== Modyfing grub config"
    sed -i "0,/menuentry/{
        /menuentry/{
            :a
            N;s/\}/\}/;T a
            p;h
            s/\('.*\)'/\1 GPD Pocket'/
            s|\(linuxefi[^\n]*\)\n|\1 nomodeset fbcon=rotate:1\n|
            p;g
            s/\('.*\)'/\1 GPD Pocket Kickstart'/
            s|\(linuxefi[^\n]*\)\n|\1 nomodeset fbcon=rotate:1 ks=hd:$volid/ks.cfg\n|
        }}" "$wd/efiboot/EFI/BOOT/grub.cfg"
fi
rsync -a "$wd/efiboot/EFI/BOOT/grub.cfg" "$wd/files/EFI/BOOT"

echo "=== Unmounting the root partition and efiboot image"
umount "$wd/efiboot"
umount "$wd/mnt"

echo "=== Making new squashfs image of the root partition"
rm -f "$wd/files/$squashfspath"
mksquashfs "$wd/squashfs/" "$wd/files/$squashfspath"

echo "=== Copying kickstart file to the LiveOS"
rsync -a ks.cfg "$wd/files"

echo "=== Creating ISO image"
xorriso -as mkisofs \
    -iso-level 3 \
    -J \
    -joliet-long \
    -full-iso9660-filenames \
    -volid "$volid" \
    -eltorito-boot "isolinux/isolinux.bin" \
    -eltorito-catalog "isolinux/boot.cat" \
    -no-emul-boot \
    -boot-load-size 4 \
    -boot-info-table \
    -isohybrid-mbr /usr/share/syslinux/isohdpfx.bin \
    -eltorito-alt-boot \
    -e images/efiboot.img \
    -no-emul-boot \
    -isohybrid-gpt-basdat \
    -output "$iso_out" \
    "$wd/files"

echo "=== Removing working directories"
rm -rf "$wd"
